@@ -6,7 +6,7 @@
 */
 /* libburn wrappers for libisoburn 
 
-   Copyright 2007 - 2016  Thomas Schmitt, <scdbackup@gmx.net> 
+   Copyright 2007 - 2017  Thomas Schmitt, <scdbackup@gmx.net> 
    Provided under GPL version 2 or later.
 */
 
@@ -343,6 +343,8 @@ int isoburn_is_intermediate_dvd_rw(struct burn_drive *d, int flag)
                 bit9= when scanning for ISO 9660 sessions on overwritable
                       media: Do not demand a valid superblock at LBA 0
                       and scan until end of medium.
+               bit10= if not bit6: accept all xattr namespaces from external 
+                                   filesystems, not only "user.".
 */ 
 static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
                                  int flag)
@@ -386,7 +388,8 @@ static int isoburn_welcome_media(struct isoburn **o, struct burn_drive *d,
  (*o)->msgs_submit= libisoburn_default_msgs_submit;
  (*o)->msgs_submit_handle= libisoburn_default_msgs_submit_handle;
  (*o)->msgs_submit_flag= libisoburn_default_msgs_submit_flag;
- iso_image_set_ignore_aclea((*o)->image, (flag >> 5 ) & 3);
+ iso_image_set_ignore_aclea((*o)->image,
+                            ((flag >> 5 ) & 3) | ((!!(flag & 1024)) << 3));
 
 #ifdef Hardcoded_cd_rW
  /* <<< A70929 : hardcoded CD-RW with fabricated -msinfo */
@@ -536,6 +539,8 @@ ex:
                 bit9= when scanning for ISO 9660 sessions on overwritable
                       media: Do not demand a valid superblock at LBA 0
                       and scan until end of medium.
+               bit10= if not bit6: accept all xattr namespaces from external
+                                   filesystems, not only "user.".
 */
 int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
                          char *adr, int flag)
@@ -562,7 +567,8 @@ int isoburn_drive_aquire(struct burn_drive_info *drive_infos[],
    goto ex;
  drive_grabbed= 1;
  ret= isoburn_welcome_media(&o, (*drive_infos)[0].drive,
-                         (flag & (8 | 16 | 32 | 64 | 128 | 512)) | !!(flag&2));
+                            (flag & (8 | 16 | 32 | 64 | 128 | 512 | 1024)) |
+                            !!(flag&2));
  if(ret<=0)
    goto ex;
 
