@@ -6,7 +6,7 @@
 /*
   libisofs related functions of libisoburn.
 
-  Copyright 2007 - 2011  Vreixo Formoso Lopes <metalpain2002@yahoo.es>
+  Copyright 2007 - 2017  Vreixo Formoso Lopes <metalpain2002@yahoo.es>
                          Thomas Schmitt <scdbackup@gmx.net>
   Provided under GPL version 2 or later.
 */
@@ -143,7 +143,7 @@ int isoburn_read_image(struct burn_drive *d,
                        struct isoburn_read_opts *read_opts, 
                        IsoImage **image)
 {
- int ret, int_num, dummy;
+ int ret, int_num, dummy, ignore_aclea= 0;
  IsoReadOpts *ropts= NULL;
  IsoReadImageFeatures *features= NULL;
  uint32_t ms_block;
@@ -188,11 +188,11 @@ create_blank_image:;
        isoburn_report_iso_error(ret, "Cannot create image", 0, "FATAL", 0);
        goto ex;
      }
-     iso_image_set_ignore_aclea(*image,
-                         (!!(read_opts->noacl)) | ((!!read_opts->noea) << 1) );
      new_image= *image;
    } else {
      /* Blank new image for the drive */
+     if(o->image != NULL)
+       ignore_aclea= iso_image_get_ignore_aclea(o->image);
      iso_image_unref(o->image);
      ret = iso_image_new("ISOIMAGE", &o->image);
      if (ret < 0) {
@@ -203,9 +203,7 @@ create_blank_image:;
        *image = o->image;
        iso_image_ref(*image); /*protects object from premature free*/
      }
-     iso_image_set_ignore_aclea(o->image,
-                         (!!(read_opts->noacl)) | ((!!read_opts->noea) << 1) );
-
+     iso_image_set_ignore_aclea(o->image, ignore_aclea);
      ret= isoburn_root_defaults(o->image, 0);
      if(ret <= 0)
        goto ex;
@@ -480,8 +478,8 @@ static int isoburn_inspect_partition(struct isoburn *o, uint32_t img_size,
    {ret= 2; goto ex;} /* No room for volume descriptors */
  offst/= 4;
  numsec/= 4;
- if(offst + numsec != img_size)
-   {ret= 2; goto ex;} /* Partition end does not match image end */
+ if(offst + numsec > img_size)
+   {ret= 2; goto ex;} /* Partition end exceeds image end */
 
  /* Is there a PVD at the partition start ? */
  ret = burn_read_data(o->drive, (off_t) (offst + 16) * (off_t) 2048,
@@ -495,8 +493,8 @@ static int isoburn_inspect_partition(struct isoburn *o, uint32_t img_size,
      || pvm->file_structure_version[0] != 1 )
    {ret= 2; goto ex;} /* failed sanity check */
 
- if(iso_read_lsb(pvm->vol_space_size, 4) + offst != img_size)
-   {ret= 2; goto ex;} /* Image ends do not match */
+ if(iso_read_lsb(pvm->vol_space_size, 4) + offst > img_size)
+   {ret= 2; goto ex;} /* Image ends do not match plausibly */
 
  /* Now it is credible. Not yet clear is whether it is acceptable. */
  o->loaded_partition_offset= offst;
@@ -670,11 +668,32 @@ ex:;
 */
 int isoburn_invalidate_iso(struct isoburn *o, int flag)
 {
+ int end_ed_found= 0, i;
+ char *head;
+
+ head= (char *) o->target_iso_head;
  /* 
   * replace CD001 with CDXX1 in PVM.
-  * I think this is enought for invalidating an iso image
   */
- strncpy((char*)o->target_iso_head + 16 * 2048 + 1, "CDXX1", 5);
+ memcpy(head + 16 * 2048 + 1, "CDXX1", 5);
+
+ /* Invalidate further ECMA-119 volume descriptors and possible UDF volume
+    recognition sequence */
+ for(i= 17 * 2048; i < 32 * 2048; i+= 2048) {
+   if(end_ed_found) {
+     if(head[i] == 0 && strncmp(head + i + 1, "BEA01", 5) == 0)
+       memcpy(head + i + 1, "BEAX1", 5);
+     else if(head[i] == 0 && strncmp(head + i + 1, "NSR", 3) == 0)
+       memcpy(head + i + 1, "NSRX", 4);
+     else if(head[i] == 0 && strncmp(head + i + 1, "TEA", 3) == 0) 
+       memcpy(head + i + 1, "TEAX", 4);
+   } else if(strncmp(head + i + 1, "CD001", 5) == 0) {
+     if(((unsigned char *) head)[i] == 0xff)
+       end_ed_found= 1;
+     memcpy(head + i + 3, "XX", 2);
+   }
+ }
+
  return isoburn_activate_session(o->drive);
 }
 
