@@ -3,7 +3,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2017 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -207,6 +207,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->displacement_sign= 0;
  m->read_fs= 0;
  m->drives_exclusive= 1;
+ m->drives_access= 1;
  m->linux_scsi_dev_family= 0;
  m->early_stdio_test= 0;
  m->cache_num_tiles= 0;
@@ -219,6 +220,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->in_charset= NULL;
  m->isofs_st_out= time(0) - 1;
  m->indev_is_exclusive= 1;
+ m->indev_access= 1;
  m->indev_off_adr[0]= 0;
  m->isofs_st_in= 0;
  m->volset_change_pending= 0;
@@ -231,6 +233,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->out_charset= NULL;
  m->dev_fd_1= -1;
  m->outdev_is_exclusive= 1;
+ m->outdev_access= 1;
  m->outdev_off_adr[0]= 0;
  m->grow_blindly_msc2= -1;
  m->ban_stdio_write= 0;
@@ -290,11 +293,15 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    m->appended_partitions[i]= NULL;
    m->appended_part_types[i]= 0;
+   memset(m->appended_part_type_guids[i], 0, 16);
+   m->appended_part_gpt_flags[i]= 0;
  }
  m->appended_as_gpt= 0;
  m->appended_as_apm= 0;
  m->part_like_isohybrid= 0;
  m->iso_mbr_part_type= -1;
+ memset(m->iso_gpt_type_guid, 0, 16);
+ m->iso_mbr_part_flag= 0;
  memset(m->gpt_guid, 0, 16);
  m->gpt_guid_mode= 0;
  m->ascii_disc_label[0]= 0;
@@ -308,6 +315,8 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  m->vol_effective_time= 0;
  m->vol_uuid[0]= 0;
  m->all_file_dates[0]= 0;
+ m->do_override_now_time= 0;
+ m->now_time_override= 0;
 
 #ifdef Xorriso_with_libjtE
  m->libjte_handle= NULL;
@@ -469,6 +478,7 @@ int Xorriso_new(struct XorrisO ** xorriso,char *progname, int flag)
  if(strcmp(leafname, "osirrox")==0) {
    m->allow_restore= 1;
    m->drives_exclusive= 0;
+   m->drives_access= 0;
  } else if(strcmp(leafname, "xorrisofs")==0 || strcmp(leafname, "genisofs")==0 ||
         strcmp(leafname, "mkisofs")==0 || strcmp(leafname, "genisoimage")==0) {
    m->argument_emulation= 1;
