@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -47,6 +47,8 @@ int Xorriso_set_ignore_aclea(struct XorrisO *xorriso, int flag)
  hflag= (~xorriso->do_aaip) & 1;
  if((xorriso->ino_behavior & (1 | 2)) && !(xorriso->do_aaip & (4 | 16)))
    hflag|= 2; 
+ if(xorriso->do_aaip & 1024)
+   hflag|= 8;
  iso_image_set_ignore_aclea(volume, hflag);
  return(1);
 }
@@ -625,7 +627,7 @@ int Xorriso_boot_status_non_mbr(struct XorrisO *xorriso, IsoImage *image,
 int Xorriso_append_part_status(struct XorrisO *xorriso, IsoImage *image,
                              char *filter, FILE *fp, int flag)
 {
- int i, is_default;
+ int i, l, is_default;
 
  is_default= (xorriso->appended_as_gpt == 0);
  sprintf(xorriso->result_line, "-boot_image any appended_part_as=%s\n",
@@ -635,8 +637,16 @@ int Xorriso_append_part_status(struct XorrisO *xorriso, IsoImage *image,
  for(i= 0; i < Xorriso_max_appended_partitionS; i++) {
    if(xorriso->appended_partitions[i] == NULL)
  continue;
-   sprintf(xorriso->result_line, "-append_partition %d 0x%2.2x ",
-           i + 1, (unsigned int) xorriso->appended_part_types[i]);
+   sprintf(xorriso->result_line, "-append_partition %d ", i + 1);
+   l= strlen(xorriso->result_line);
+   if(xorriso->appended_part_gpt_flags[i] & 1) {
+     Xorriso__format_guid(xorriso->appended_part_type_guids[i],
+                          xorriso->result_line + l, 0);
+     strcpy(xorriso->result_line + l + 32, " ");
+   } else {
+     sprintf(xorriso->result_line + l, "0x%2.2x ",
+             (unsigned int) xorriso->appended_part_types[i]);
+   }
    Text_shellsafe(xorriso->appended_partitions[i], xorriso->result_line, 1);
    strcat(xorriso->result_line, "\n");
    Xorriso_status_result(xorriso, filter, fp, flag & 2);
@@ -1646,25 +1656,32 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
  int appended_as_gpt= 0, have_prep= 0, did_sysarea= 0, cared_for_apm= 0;
  int cared_for_sparc= 0, have_hfsplus= 0;
  int have_sysarea= 0, ptable_killer, imported_iso, have_alpha_ldr_path= 0;
- int have_protective_msdos= 0, appended_partition= 0, part_like_isohybrid= 0;
- int iso_mbr_part_type= -1;
+ int have_protective_msdos= 0, part_like_isohybrid= 0;
+
+#ifdef Not_any_more_because_padding_is_now_after_partitions
+ int appended_partition= 0;
+#endif
+
+ int iso_mbr_part_type= -1, iso_gpt_part_idx= -1;
  unsigned int prev_pltf= 0;
  unsigned long int sa_options= 0, partno, id_tag, perms, start_cyl, num_blocks;
  unsigned long int part_status, part_type, start_block, partition_offset= 0;
  uint32_t high_block= 0;
- char name[24], *textpt, *contentpt, *buf= NULL;
+ char name[24], *textpt, *contentpt, *buf= NULL, part_type_text[37];
  char **lines= NULL;
  double num[8];
  char *cat_path= "";
  struct elto_img_par *et_imgs= NULL;
  int elto_count= 0;
- uint32_t img_blocks= 0, mbr_parts_end= 0;
+ uint32_t img_blocks= 0, mbr_parts_end= 0, iso_part_blocks;
  struct FindjoB *job= NULL;
  struct stat dir_stbuf;
  IsoImage *image;
  char *volid, *crt, *mdt, *ext, *eft, uuid[17], *uuid_time;
  char **app_pseudo_paths= NULL;
  struct tm tm_erg;
+ int was_force_bootable= 0;
+ uint64_t gpt_bheader_block= 0;
 
  struct mbr_par {
    uint8_t ptype;
@@ -1678,6 +1695,7 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
 
  struct gpt_par {
    int ptype; /* 0= unknown, 1= gpt-basdat, 2=gpt-hfsplus, 3=EFI */
+   uint8_t type_guid[16];
    int is_gap;
    int has_path;
    char *path;
@@ -1920,6 +1938,9 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      idx= num[0] - 1;
      mbrpts[idx].has_path= 1;
 
+   } else if(strcmp(name, "GPT lba range      :") == 0) {
+     gpt_bheader_block= num[2];
+
    } else if(strcmp(name, "GPT type GUID      :") == 0) {
      idx= num[0] - 1;
      if(strcmp(textpt, "a2a0d0ebe5b9334487c068b6b72699c7") == 0)
@@ -1930,13 +1951,18 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
        gpts[idx].ptype= 3; /* EFI System Partition */
      else
        gpts[idx].ptype= 0;
+     Xorriso_parse_guid(xorriso, textpt, gpts[idx].type_guid, 1);
 
    } else if(strcmp(name, "GPT start and size :") == 0) {
      idx= num[0] - 1;
      if(num[2] > 0)
        appended_as_gpt= 1;
-     gpts[idx].start_block= num[1];
-     gpts[idx].block_count= num[2];
+     start_block= gpts[idx].start_block= num[1];
+     num_blocks= gpts[idx].block_count= num[2];
+     if(start_block == partition_offset * 4 &&
+        (start_block + num_blocks) >= high_block * 4 &&
+        iso_gpt_part_idx < 0)
+       iso_gpt_part_idx= idx;
 
    } else if(strcmp(name, "GPT partition path :") == 0) {
      idx= num[0] - 1;
@@ -1968,8 +1994,23 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
    }
  }
 
- if(!have_protective_msdos)
-   appended_as_gpt= 0;
+ if(appended_as_gpt && !have_protective_msdos) {
+   if(mbr_count != 1) {
+     appended_as_gpt= 0;
+   } else if(mbrpts[0].ptype != 0xee || mbrpts[0].start_block != 1) {
+     appended_as_gpt= 0;
+   } else if(gpt_bheader_block != mbrpts[0].block_count) {
+     appended_as_gpt= 0;
+   }
+ }
+
+ iso_part_blocks= img_blocks;
+ for(mbr_idx = 0; mbr_idx < mbr_count; mbr_idx++) {
+   if(mbrpts[mbr_idx].start_block == partition_offset * 4) {
+     iso_part_blocks= mbrpts[mbr_idx].block_count + partition_offset * 4;
+ break;
+   }
+ }
 
  /* Second pass: scan for System Area info */
  for(i= 0; i < line_count; i++) {
@@ -2059,7 +2100,7 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      sscanf(contentpt, "%lu 0x%lx 0x%lx %lu %lu",
             &partno, &part_status, &part_type, &start_block, &num_blocks);
      if(num_blocks > 0 && part_type != 0x00 && part_type != 0xee &&
-        (img_blocks <= start_block ||
+        (iso_part_blocks <= start_block ||
          (have_protective_msdos && img_blocks == mbr_parts_end &&
           partno > 1))) {
        if(!appended_as_gpt) {
@@ -2069,7 +2110,11 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
                                imported_iso);
          if(partno >= 1 && (int) partno <= mbr_count)
            mbrpts[partno - 1].appended= 1;
+
+#ifdef Not_any_more_because_padding_is_now_after_partitions
          appended_partition= 1;
+#endif
+
        }
      } else if(part_type == 0x41 && have_prep) {
        if(mkisofs) {
@@ -2080,8 +2125,13 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
        Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) start_block,
                              ((uint64_t) start_block) + num_blocks - 1, "d",
                              imported_iso);
-     } else if((part_status & 0x80) && part_type == 0x00 &&
-               start_block == 0 && num_blocks == 1) {
+     }
+     if((part_status & 0x80) && !was_force_bootable) {
+       was_force_bootable= 1;
+       if(buf[0]) {
+         Xorriso_record_cmd_linE
+         buf[0]= 0;
+       }
        if(mkisofs)
          sprintf(buf, "--mbr-force-bootable");
        else
@@ -2159,7 +2209,7 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      else
        part_type= 0xcd;
 
-     if(high_block < num[1] && num[2] > 0 && !gpts[idx].is_gap) {
+     if(high_block * 4 < num[1] && num[2] > 0 && !gpts[idx].is_gap) {
        for(mbr_idx = 0; mbr_idx < mbr_count; mbr_idx++) {
          if(mbrpts[mbr_idx].start_block == num[1]) {
            if(mbrpts[mbr_idx].block_count != num[2] && !(flag & 1)) {
@@ -2174,15 +2224,23 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
          }
        }
        if(mbr_idx >= mbr_count) {
-         if(appended_as_gpt == 1)
+         if(appended_as_gpt == 1) {
            appended_as_gpt= 2;
-         sprintf(buf, "-append_partition %d 0x%lx ", idx + 1, part_type);
+           Xorriso__format_guid(gpts[idx].type_guid, part_type_text, 0);
+         } else {
+           sprintf(part_type_text, "0x%lx", part_type);
+         }
+         sprintf(buf, "-append_partition %d %s ", idx + 1, part_type_text);
          Xorriso_add_intvl_adr(xorriso, buf, (uint64_t) num[1],
                                (uint64_t) (num[1] + num[2] - 1.0), "d",
                                imported_iso);
          Xorriso_record_cmd_linE
          buf[0]= 0;
+
+#ifdef Not_any_more_because_padding_is_now_after_partitions
          appended_partition= 1;
+#endif
+
        }
 
        /* Check for isohybri-ish MBR and GPT mix */
@@ -2444,13 +2502,23 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
      sprintf(buf, "-boot_image any iso_mbr_part_type=0x%2.2x",
                   (unsigned int) iso_mbr_part_type);
    Xorriso_record_cmd_linE
+
+ } else if(iso_gpt_part_idx >= 0) {
+   if(mkisofs)
+     sprintf(buf, "-iso_mbr_part_type ");
+   else
+     sprintf(buf, "-boot_image any iso_mbr_part_type=");
+   Xorriso__format_guid(gpts[iso_gpt_part_idx].type_guid, buf + strlen(buf),
+                        0);
+   Xorriso_record_cmd_linE
+   
  }
 
  /* Issue commands related to El Torito */
  if(elto_count <= 0)
    goto after_el_torito;
 
- if(efi_boot_part) {
+ if(efi_boot_part == 1) {
    if(mkisofs)
      sprintf(buf, "-efi-boot-part --efi-boot-image");
    else
@@ -2510,7 +2578,11 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
                                  mbrpts[i].block_count - 1,
                                  "d", imported_iso);
            Xorriso_record_cmd_linE
+
+#ifdef Not_any_more_because_padding_is_now_after_partitions
            appended_partition= 1;
+#endif
+
            buf[0]= 0;
          }
        }
@@ -2521,7 +2593,7 @@ static int Xorriso_scan_report_lines(struct XorrisO *xorriso,
                (unsigned long) mbrpts[i].block_count);
        et_imgs[idx].path= app_pseudo_paths[idx];
      }
-     if (et_imgs[idx].path[0] == 0) {
+     if (et_imgs[idx].path[0] == 0 && efi_boot_part != 2) {
        for(i= 0; i < gpt_count; i++) {
          if(have_protective_msdos && (
             gpts[i].start_block == ((uint64_t) et_imgs[idx].lba) * 4 &&
@@ -2658,6 +2730,8 @@ after_el_torito:
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "SORRY", 0);
  }
 
+#ifdef Not_any_more_because_padding_is_now_after_partitions
+
  if(appended_partition) {
    if(mkisofs)
      sprintf(buf, "-no-pad");
@@ -2666,6 +2740,8 @@ after_el_torito:
    Xorriso_record_cmd_linE
  }
 
+#endif /* Not_any_more_because_padding_is_now_after_partitions */
+
  ret= 1;
 ex:
  xorriso->show_hfs_cmds= NULL;
