@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2015 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -36,8 +36,8 @@ int Xorriso__append_boot_params(char *line, ElToritoBootImage *bootimg,
 int Xorriso_get_volume(struct XorrisO *xorriso, IsoImage **volume,
                        int flag);
 
-
-
+int Xorriso_record_cmd_line(struct XorrisO *xorriso, char *buf,
+                            char **cmds, int *cmd_count, int flag);
 
 
 #endif /* ! Xorriso_pvt_iso_img_includeD */
