@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -495,7 +495,7 @@ int Xorriso_path_from_lba(struct XorrisO *xorriso, IsoNode *node, int lba,
 
 /* @param flag bit0= in_node is valid, do not resolve iso_adr
                bit2= recognize and parse split parts despite
-                     xorrio->split_size <= 0
+                     xorriso->split_size <= 0
 */
 int Xorriso_identify_split(struct XorrisO *xorriso, char *iso_adr,
                            void *in_node,
@@ -627,6 +627,8 @@ ex:;
 
 /* @param flag bit0= node is valid, do not resolve path
                bit1= insist in complete collection of part files
+               bit2= recognize and parse split parts despite
+                     xorriso->split_size <= 0
 */
 int Xorriso_is_split(struct XorrisO *xorriso, char *path, void *node,
                      int flag)
@@ -636,7 +638,7 @@ int Xorriso_is_split(struct XorrisO *xorriso, char *path, void *node,
  struct stat stbuf;
 
  ret= Xorriso_identify_split(xorriso, path, node, &split_parts,
-                             &split_count, &stbuf, flag & 3);
+                             &split_count, &stbuf, flag & 7);
  if(split_parts!=NULL)
    Splitparts_destroy(&split_parts, split_count, 0);
  return(ret>0);
@@ -813,7 +815,8 @@ too_long:
 /*
    @param flag      bit0= do not report to result but only retrieve attr text
                     bit1= path is disk_path
-                    bit3= do not ignore eventual non-user attributes.
+                    bit3= do not ignore non-user attributes
+                    bit4= ignore isofs. attributes despite bit3
                     bit5= in case of symbolic link on disk: inquire link target
                     bit6= check for existence of xattr, return 0 or 1
                           (depends also on bit3)
@@ -827,6 +830,8 @@ int Xorriso_getfattr(struct XorrisO *xorriso, void *in_node, char *path,
 
  if(attr_text != NULL)
    *attr_text= NULL;
+ if((xorriso->do_aaip & 1024) && !(flag & 8))
+   flag|= 8 | 16;
  ret= Xorriso_get_attrs(xorriso, in_node, path, &num_attrs, &names,
                         &value_lengths, &values, flag & (2 | 8 | 32));
  if(ret <= 0)
@@ -852,6 +857,9 @@ int Xorriso_getfattr(struct XorrisO *xorriso, void *in_node, char *path,
      }
    }
    for(i= 0; i < (int) num_attrs; i++) {
+     if(flag & 16)
+       if(strncmp(names[i], "isofs.", 6) == 0)
+   continue;
      if(strlen(names[i]) + value_lengths[i] >= SfileadrL) {
        sprintf(xorriso->result_line, "# oversized: name %d , value %d bytes\n",
                (int) strlen(names[i]), (int) value_lengths[i]);
@@ -880,6 +888,14 @@ int Xorriso_getfattr(struct XorrisO *xorriso, void *in_node, char *path,
      }
    }
  }
+ if((flag & 16) && attr_text != NULL)
+   if(*attr_text != NULL)
+     if((*attr_text)[0] == 0) {
+       free(*attr_text);
+       *attr_text= NULL;
+       ret= 2;
+       goto ex;
+     }
  if(!(flag & 1)) {
    strcpy(xorriso->result_line, "\n");
    Xorriso_result(xorriso, 0);
@@ -1386,6 +1402,8 @@ int Xorriso_make_md5(struct XorrisO *xorriso, void *in_node, char *path,
 
 
 /* @param flag bit0= do not only sum up sizes but also print subdirs
+
+               bit2= do not report result by Xorriso_result()
 */
 int Xorriso_show_du_subs(struct XorrisO *xorriso, IsoDir *dir_node,
                       char *abs_path, char *rel_path, off_t *size,
@@ -1502,13 +1520,15 @@ much_too_long:;
    report_size= sub_size/1024;
    if(report_size*1024<sub_size)
       report_size++;
-   if(xorriso->sh_style_result)
-     sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
-   else
-     sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
-   sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
-           Xorriso_esc_filepath(xorriso, show_path, sfe, 0));
-   Xorriso_result(xorriso, 0);
+   if(!(flag & 4)) {
+     if(xorriso->sh_style_result)
+       sprintf(xorriso->result_line, "%-7.f ",(double) (report_size));
+     else
+       sprintf(xorriso->result_line, "%7.f ",(double) (report_size));
+     sprintf(xorriso->result_line+strlen(xorriso->result_line), "%s\n",
+             Xorriso_esc_filepath(xorriso, show_path, sfe, 0));
+     Xorriso_result(xorriso, 0);
+   }
  }
  ret= 1;
 ex:;
@@ -1830,6 +1850,41 @@ ex:;
 }
 
 
+/*
+   @return: <=0 = error, 1= directory tree, 2= path leads to non-directory 
+*/
+int Xorriso_get_dus(struct XorrisO *xorriso, char *iso_rr_path, off_t *size,
+                    off_t boss_mem, int flag)
+{
+ int ret;
+ IsoNode *node;
+ char *path= NULL;
+ struct stat stbuf;
+
+ Xorriso_alloc_meM(path, char, SfileadrL);
+
+ ret= Xorriso_make_abs_adr(xorriso, xorriso->wdi, iso_rr_path, path,
+                           1 | 2 | 4);
+ if(ret <= 0)
+   goto ex;
+ ret= Xorriso_fake_stbuf(xorriso, path, &stbuf, &node, 0);
+ if(ret <= 0)
+   goto ex;
+ if(!S_ISDIR(stbuf.st_mode)) {
+   *size= stbuf.st_size;
+   ret= 2; goto ex;
+ }
+ ret= Xorriso_show_du_subs(xorriso, (IsoDir *) node, path, iso_rr_path, size,
+                           boss_mem, 4);
+ if(ret <= 0)
+   goto ex;
+ ret= 1;
+ex:
+ Xorriso_free_meM(path);
+ return(ret);
+}
+
+
 /* This function needs less buffer memory than Xorriso_ls_filev() but cannot
    perform structured pattern matching as done by Xorriso_expand_pattern()
    for subsequent Xorriso_ls_filev().
@@ -2024,6 +2079,7 @@ ex:;
 /* @param flag bit0= a match count !=1 is a FAILURE event
                bit1= with bit0 tolerate 0 matches if pattern is a constant
                bit2= do not issue debug messages about temporay memory needs
+               bit3= do not add unresolved pattern to filev
 */
 int Xorriso_expand_pattern(struct XorrisO *xorriso,
                            int num_patterns, char **patterns, int extra_filec,
@@ -2092,7 +2148,8 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
                                        mem, &dive_count, 1 | abs_adr);
    if(ret<=0)
      goto ex;
-   if(was_count==count && strcmp(patterns[i],"*")!=0 && (flag&3)!=1) {
+   if(was_count==count && strcmp(patterns[i],"*")!=0 && (flag&3)!=1 &&
+      !(flag & 8)) {
      count++;
      Xorriso_eval_nonmatch(xorriso, patterns[i], &nonconst_mismatches, mem, 0);
    }
@@ -2105,7 +2162,7 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
  count+= extra_filec;
  (*mem)+= extra_filec * sizeof(char *);
  if(count<=0)
-   {ret= 0; goto ex;}
+   {ret= !!(flag & 8); goto ex;}
  ret= Xorriso_alloc_pattern_mem(xorriso, *mem, count, filev, !!(flag & 4));
  if(ret<=0)
    goto ex;
@@ -2122,7 +2179,8 @@ int Xorriso_expand_pattern(struct XorrisO *xorriso,
                                        mem, &dive_count, abs_adr);
    if(ret<=0)
      goto ex;
-   if(was_filec == *filec && strcmp(patterns[i],"*")!=0) {
+   if(was_filec == *filec && strcmp(patterns[i],"*")!=0 && (flag&3) != 1 &&
+      !(flag & 8)) {
      (*filev)[*filec]= strdup(patterns[i]);
      if((*filev)[*filec]==NULL) {
        (*mem)= strlen(patterns[i])+1;
