@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2014 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -124,7 +124,7 @@ int Xorriso_startup_libraries(struct XorrisO *xorriso, int flag)
 
 
 /* First an ugly compile time check for header version compatibility.
-   If everthing matches, then no C code is produced. In case of mismatch,
+   If everything matches, then no C code is produced. In case of mismatch,
    intentionally faulty C code will be inserted.
 */
 
@@ -526,7 +526,7 @@ int Xorriso_process_msg_queues(struct XorrisO *xorriso, int flag)
  ret= pthread_mutex_lock(&(xorriso->lib_msg_queue_lock));
  if(ret != 0) {
    Xorriso_msgs_submit(xorriso, 0,
-              "Cannot aquire mutex lock for processing library message queues",
+             "Cannot acquire mutex lock for processing library message queues",
                ret, "FATAL", 0);
  } else
    locked= 1;
@@ -980,3 +980,39 @@ int Xorriso_set_data_cache(struct XorrisO *xorriso, void *o,
 }
 
 
+int Xorriso_format_guid(struct XorrisO *xorriso, uint8_t guid[16], char *line,
+                        int flag)
+{
+
+ /* >>> Maybe let the user switch between hex string and structured text */;
+
+ Xorriso__format_guid(guid, line, 1);
+ return(1);
+}
+
+
+int Xorriso_make_guid(struct XorrisO *xorriso, char *line, int flag)
+{
+ uint8_t guid[16];
+ int ret;
+
+ iso_generate_gpt_guid(guid);
+ ret= Xorriso_format_guid(xorriso, guid, line, 0);
+ return(ret);
+}
+
+
+int Xorriso_set_libisofs_now(struct XorrisO *xorriso, int flag)
+{
+ int ret;
+ time_t now;
+
+ if(xorriso->do_override_now_time) {
+   now= xorriso->now_time_override;
+   ret= iso_nowtime(&now, 1);
+ } else {
+   ret= iso_nowtime(&now, 0);
+ }
+ return(ret);
+}
+
