@@ -6,6 +6,7 @@
    Specialized converter from xorriso/xorriso.texi to xorriso/xorriso.1,
    or from xorriso/xorrisofs.texi to xorriso/xorrisofs.1,
    or from xorriso/xorrecord.texi to xorriso/xorrecord.1.
+   or from xorriso/xorriso-tcltk.texi to xorriso/xorriso-tcltk.1.
 
    The conversion rules are described at the beginning of xorriso/xorriso.texi
 
@@ -475,6 +476,7 @@ usage:;
    fprintf(stderr, "  -filter     stdin                -> stdout\n");
    fprintf(stderr, "  -xorrisofs  process xorriso/xorrisofs.texi\n");
    fprintf(stderr, "  -xorrecord  process xorriso/xorrecord.texi\n");
+   fprintf(stderr, "  -xorriso-tcltk  process xorriso/-xorriso-tcltk.texi\n");
    exit(2);
  }
  for(i= 1; i < argc; i++) {
@@ -488,6 +490,9 @@ usage:;
    } else if(strcmp(argv[i], "-xorrecord") == 0) {
      strcpy(name_in, "xorriso/xorrecord.texi");
      strcpy(name_out, "xorriso/xorrecord.1");
+   } else if(strcmp(argv[i], "-xorriso-tcltk") == 0) {
+     strcpy(name_in, "xorriso/xorriso-tcltk.texi");
+     strcpy(name_out, "xorriso/xorriso-tcltk.1");
    } else {
      fprintf(stderr, "%s : unknown option %s\n", argv[0], argv[i]);
      goto usage;
