@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -14,6 +14,14 @@
 
 #include <regex.h>
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#else
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+#endif
+
 
 char *Text_shellsafe(char *in_text, char *out_text, int flag);
 
@@ -101,5 +109,10 @@ int Xorriso__to_upper(char *in, char *out, int out_size, int flag);
 int Xorriso__exchange_prefix(char *source_prefix, char *target_prefix,
                              char *eff_source, char *eff_target, int flag);
 
+/* @param text  takes result, must provide at least 37 characters of storage
+   @param flag  bit0= structured text format (else hex string)
+*/
+int Xorriso__format_guid(uint8_t guid[16], char *text, int flag);
+
 #endif /* ! Xorriso_pvt_misc_includeD */
 
