@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2017 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2018 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -218,8 +218,7 @@ ex:;
  xorriso->allow_graft_points= allow_graft_points_mem;
  Sfile_make_argv("", "", &argc, &argv, 2);
  Xorriso_free_meM(pathspec);
- if(flag & 1)
-   Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
+ Xorriso_read_lines(xorriso, fp, &linecount, &argc, &argv, 2);
 
  if(fp != NULL && fp != stdin)
    fclose(fp);
@@ -694,7 +693,7 @@ int Xorriso_option_rollback(struct XorrisO *xorriso, int flag)
  ret= Xorriso_give_up_drive(xorriso, 1|8);
  if(ret<=0)
    goto ex;
- xorriso->image_start_mode&= ~(1<<31); /* reactivate eventual -load address */
+ xorriso->image_start_mode&= ~(1u<<31); /* reactivate eventual -load address */
  ret= Xorriso_option_dev(xorriso, indev, 1|4);
 ex:;
  Xorriso_free_meM(indev);
@@ -831,12 +830,13 @@ int Xorriso_option_session_log(struct XorrisO *xorriso, char *path, int flag)
 int Xorriso_option_setfacl_listi(struct XorrisO *xorriso, char *path, int flag)
 {
  int ret, eaten, line_size;
- size_t buf_size= 0, buf_add= 64 * 1024, l, linecount= 0;
+ size_t buf_size= 0, buf_add, l, linecount= 0, diff_buf_wpt;
  char *line= NULL, *buf= NULL, *wpt, *new_buf, limit_text[80];
  char *file_path= NULL, *uid= NULL, *gid= NULL;
  FILE *fp= NULL;
 
  line_size= SfileadrL * 4;
+ buf_add= line_size * 4;
  Xorriso_alloc_meM(line, char, line_size);
  Xorriso_alloc_meM(file_path, char, SfileadrL);
  Xorriso_alloc_meM(uid, char, 161);
@@ -923,11 +923,13 @@ int Xorriso_option_setfacl_listi(struct XorrisO *xorriso, char *path, int flag)
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
        ret= 0; goto ex;
      }
+     diff_buf_wpt= wpt - buf;
      buf_size+= buf_add;
      new_buf= realloc(buf, buf_size);
      if(new_buf == NULL)
        goto out_of_mem;
      buf= new_buf;
+     wpt= buf + diff_buf_wpt;
    }
    memcpy(wpt, line, l);
    *(wpt + l)= '\n';
@@ -989,9 +991,11 @@ int Xorriso_option_setfacli(struct XorrisO *xorriso, char *acl_text,
  ret= Xorriso_normalize_acl_text(xorriso, acl_text,
                                  &access_acl_text, &default_acl_text, 0);
  if(access_acl_text != NULL && default_acl_text != NULL) {
-   sprintf(xorriso->info_text, "Access-ACL :\n%s", access_acl_text);
+   strcpy(xorriso->info_text, "Access-ACL :\n");
+   Xorriso_set_info_text(xorriso, access_acl_text, 2000, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
-   sprintf(xorriso->info_text, "Default-ACL :\n%s", default_acl_text);
+   strcpy(xorriso->info_text, "Default-ACL :\n");
+   Xorriso_set_info_text(xorriso, default_acl_text, 2000, 1);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
  } else if(access_acl_text == NULL && default_acl_text == NULL) {
    sprintf(xorriso->info_text, "Will delete Access-ACL and Default-ACL");
@@ -1989,7 +1993,7 @@ int Xorriso_option_version(struct XorrisO *xorriso, int flag)
  sprintf(xorriso->result_line,
 "ISO 9660 Rock Ridge filesystem manipulator and CD/DVD/BD burn program\n");
  sprintf(xorriso->result_line+strlen(xorriso->result_line),
-"Copyright (C) 2017, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
+"Copyright (C) 2019, Thomas Schmitt <scdbackup@gmx.net>, libburnia project.\n");
  Xorriso_result(xorriso, 0);
  sprintf(xorriso->result_line,
          "xorriso version   :  %d.%d.%d%s\n",
@@ -2189,11 +2193,14 @@ int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag)
 {
  int ret;
 
- if(strcmp(mode, "off")==0)
-   xorriso->do_aaip&= ~12;
- else if(strcmp(mode, "on")==0)
+ if(strcmp(mode, "off") == 0) {
+   xorriso->do_aaip&= ~(4 | 8 | 1024);
+ } else if(strcmp(mode, "on") == 0 || strcmp(mode, "user") == 0) {
+   xorriso->do_aaip&= ~1024;
    xorriso->do_aaip|= (4 | 8);
- else {
+ } else if(strcmp(mode, "any") == 0) {
+   xorriso->do_aaip|= (4 | 8 | 1024);
+ } else {
    sprintf(xorriso->info_text, "-xattr: unknown mode '%s'", mode);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
