@@ -105,6 +105,9 @@ int Xorriso_named_pipe_loop(struct XorrisO *xorriso,
 char *Xorriso_esc_filepath(struct XorrisO *xorriso,
                            char *in_text, char *out_text, int flag);
 
+int Xorriso_set_info_text(struct XorrisO *xorriso, char *text,
+                          size_t trunc_len, int flag);
+
 #ifdef Xorriso_with_editlinE
 int Xorriso__shutdown_editline(int flag);
 #endif
