@@ -2,7 +2,7 @@
 /* xorriso - libisoburn higher level API which creates, loads, manipulates
              and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2016 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -73,8 +73,8 @@ struct XorrisO;
     @since 0.5.8
 */
 #define Xorriso_header_version_majoR  1
-#define Xorriso_header_version_minoR  4
-#define Xorriso_header_version_micrO  6
+#define Xorriso_header_version_minoR  5
+#define Xorriso_header_version_micrO  2
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -820,6 +820,9 @@ int Xorriso_sieve_dispose(struct XorrisO *xorriso, int flag);
                      (Note: prefix is "Creation Time: ")
     "DVD obs 64 kB:" up to 1 result from -list_extras
                      argv[0]= "yes" or "no"
+    "Drive access: " up to 2 result from -dev, -indev, -toc
+                     argv[0]= "exclusive", "shared"
+                     argv[1]= "readonly","restricted"
     "Drive current:" up to 2 results from -dev, -indev, -toc, others
                      argv[0]= command ("-dev", "-outdev", "-indev")
                      argv[1]= drive address
@@ -1502,6 +1505,10 @@ int Xorriso_option_disk_pattern(struct XorrisO *xorriso, char *mode, int flag);
 int Xorriso_option_displacement(struct XorrisO *xorriso, char *value,
                                 int flag);
 
+/* Command -drive_access "exclusive"|"shared":"readonly"|"unrestricted" */
+/* @since 1.5.2 */
+int Xorriso_option_drive_access(struct XorrisO *xorriso, char *mode, int flag);
+
 /* Command -drive_class */
 /* @since 0.3.2 */
 int Xorriso_option_drive_class(struct XorrisO *xorriso,
@@ -1593,10 +1600,13 @@ int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag);
 /* Command -for_backup is a shortcut for
      Xorriso_option_hardlinks(xorriso, "on", 0);
      Xorriso_option_acl(xorriso, "on", 0);
-     Xorriso_option_xattr(xorriso, "on", 0);
+     Xorriso_option_xattr(xorriso, "any", 0);
      Xorriso_option_md5(xorriso, "on", 0);
 */
 /* @since 0.4.0 */
+/* xattr "any"
+   @since 1.5.0
+*/
 
 /* Command -fs */
 /* @since 0.1.0 */
@@ -1642,6 +1652,10 @@ int Xorriso_option_hide(struct XorrisO *xorriso, char *hide_state,
 /* @since 0.1.0 */
 int Xorriso_option_history(struct XorrisO *xorriso, char *line, int flag);
 
+/* Command -iso_nowtime "dynamic"|timespec */
+/* @since 1.5.2 */
+int Xorriso_option_iso_nowtime(struct XorrisO *xorriso, char *text, int flag);
+
 /* Command -iso_rr_pattern "on"|"ls"|"off" */
 /* @since 0.1.0 */
 int Xorriso_option_iso_rr_pattern(struct XorrisO *xorriso, char *mode,
@@ -1741,6 +1755,8 @@ int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
                              1= -compare_l
                              2= -update_l
                              3= -extract_l
+                             4= -update_lxi  @since 1.4.8
+                             5= -update_li   @since 1.4.8
 */
 /* @since 0.2.0 */
 int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
