@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2013 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -34,6 +34,9 @@
 #endif
 #endif
 
+/* for pthread_mutex_t on OpenBSD */
+#include <pthread.h>
+
 
 /** The source code release timestamp */
 #include "xorriso_timestamp.h"
@@ -146,6 +149,7 @@ struct XorrisO { /* the global context of xorriso */
  int do_fat;
  int do_rockridge;
  int do_iso1999;
+ int ecma119_map;     /* 0=unmapped , 1=stripped , 2=uppercase , 3=lowercase */
 
  int do_aaip; /* bit0= ACL in
                  bit1= ACL out
@@ -156,7 +160,11 @@ struct XorrisO { /* the global context of xorriso */
                  bit6= omit content check if bit5 check is conclusive
                  bit7= omit dev check with bit5
                  bit8= store output charset in xattr "isofs.cs"
-                 bit9= allow to set input charset from xattr "isofs.cs"
+                 bit9= set input charset from xattr "isofs.cs"
+                bit10= if bit2: import from local filesystem all xattr
+                                namespaces, not only "user."
+                       if bit3: export to local filesystem all xattr
+                                namespaces, not only "user."
               */
 
  int do_md5;  /* bit0= read MD5 array
@@ -190,6 +198,9 @@ struct XorrisO { /* the global context of xorriso */
  int do_follow_param;
  int do_follow_links;
  int follow_link_limit;
+ int resolve_link_rec_count;
+ int resolve_link_rec_limit;
+ int do_follow_concat;
  int do_follow_mount;
  int do_global_uid;
  uid_t global_uid;
@@ -274,7 +285,12 @@ struct XorrisO { /* the global context of xorriso */
  uint32_t displacement;
  int displacement_sign;
 
+ int read_fs;            /* bit0=norock , bit1=nojoliet */
+
  int drives_exclusive;   /* burn_preset_device_open() param exclusive */
+ int drives_access;      /* 0=readonly , 1=unrestricted */
+
+ int linux_scsi_dev_family; /* 0= default, 1= sr, 2= scd, (3= st), 4= sg */
 
  int early_stdio_test;   /* For burn_allow_drive_role_4():
                             bit1= Test whether a stdio drive can be opened for
@@ -292,15 +308,16 @@ struct XorrisO { /* the global context of xorriso */
  int cache_tile_blocks;
  int cache_default;       /* bit0= cache_num_tiles, bit1= cache_tile_blocks */
 
- int do_calm_drive;      /* bit0= calm down drive after aquiring it */
+ int do_calm_drive;      /* bit0= calm down drive after acquiring it */
 
  char indev[SfileadrL];
  void *in_drive_handle;  /* interpreted only by libburnia oriented modules */
  void *in_volset_handle; /* interpreted only by libburnia oriented modules */
  char *in_charset;       /* The charset to interpret the filename bytes */
  int indev_is_exclusive;
+ int indev_access;              /* see drives_access */
  char indev_off_adr[SfileadrL]; /* Result of burn_drive_convert_fs_adr(indev)
-                                   when indev gets aquired. */
+                                   when indev gets acquired. */
 
  time_t isofs_st_out;    /* A time point at least 1 second before image
                             composition began. To be stored with image as
@@ -309,9 +326,12 @@ struct XorrisO { /* the global context of xorriso */
                             loaded image. */
 
  int volset_change_pending; /* whether -commit would make sense
-                               0= no change pending , 1= change pending
+                               0= no change pending
+                               1= change pending
                                2= change pending, but -as misofs -print-size
                                   was performed on the changed image model
+                               3= change pending, but the attempt to write it
+                                  failed
                              */
  int no_volset_present;     /* set to 1 on first failure */
 
@@ -327,8 +347,9 @@ struct XorrisO { /* the global context of xorriso */
                      connected to externaly perveived stdout.
                   */
  int outdev_is_exclusive;
+ int outdev_access;              /* see drives_access */
  char outdev_off_adr[SfileadrL]; /* Result of burn_drive_convert_fs_adr(outdev)
-		        	    when outdev gets aquired. */
+		        	    when outdev gets acquired. */
 
  int grow_blindly_msc2; /* if >= 0 this causes growing from drive to drive.
                            The value is used as block address offset for
@@ -338,7 +359,12 @@ struct XorrisO { /* the global context of xorriso */
  int ban_stdio_write;
  int do_dummy;
  int do_close;
- int speed;     /* in libburn units : 1000 bytes/second , 0 = Max, -1 = Min */
+ int auto_close;     /* Whether to let do_close depend on media state */
+
+ int write_speed;    /* Write speed in libburn units : 1000 bytes/second ,
+                                           0 = Max, -1 = Min, -2= do not set */
+ int read_speed;     /* Read speed. See above */
+
  int fs;        /* fifo size in 2048 byte chunks : at most 1 GB */
  int padding;   /* number of bytes to add after ISO 9660 image */
  int do_padding_by_libisofs; /* 0= by libburn , 1= by libisofs */
@@ -353,7 +379,22 @@ struct XorrisO { /* the global context of xorriso */
                              >=16 means yes with number as start LBA  */
 
  int dvd_obs;                 /* DVD write chunk size: 0, 32k or 64k */
+
+ int modesty_on_drive;        /* "enable" of burn_drive_set_buffer_waiting()
+                                 0= disable , 1= enable waiting ,
+                                 (-1 = do not change setting)
+                               */
+ int min_buffer_usec;         /* The other parameters for this function */
+ int max_buffer_usec;
+ int buffer_timeout_sec;
+ int min_buffer_percent;
+ int max_buffer_percent;
+
+ int use_immed_bit;           /* 1= on , 0= default , -1 = off */
+ int use_immed_bit_default;   /* 1= on , 0= yet undefined , -1 = off */
+
  int stdio_sync;              /* stdio fsync interval: -1, 0, >=32 */
+ int stdio_sync_is_default;   /* 1= is still default , 0= has been set */
 
  int keep_boot_image;
  char boot_image_cat_path[SfileadrL];
@@ -376,6 +417,8 @@ struct XorrisO { /* the global context of xorriso */
                       */
  int boot_emul_default;  /* 1= boot_image_emul is still default */
  off_t boot_image_load_size;
+ int boot_img_size_default;  /* 1= boot_image_load_size is still default */
+ int boot_img_full_size;  /* 1= override boot_image_load_size by image size */
  unsigned char boot_id_string[29];
  unsigned char boot_selection_crit[21];
 
@@ -385,6 +428,7 @@ struct XorrisO { /* the global context of xorriso */
                           1= appy --efi-boot parameters when attaching to img */
 
  char system_area_disk_path[SfileadrL];
+ int system_area_clear_loaded;
  int system_area_options;             /* bit0= "GRUB protective msdos label"
                                                (a simple partition table)
                                          bit1= isohybrid boot image pointer
@@ -395,11 +439,16 @@ struct XorrisO { /* the global context of xorriso */
                                                1= MIPS Big Endian Volume Header
                                                2= MIPS Little Endian Boot Block
                                                3= SUN Disk Label for SUN SPARC
+                                               4= HP-PA PALO boot sector
+                                                  header version 4
+                                               5= HP-PA PALO boot sector
+                                                  header version 5
                                          bit8-9= Only with System area type 0
                                                Cylinder alignment mode
                                                0 = auto (align if bit1)
                                                1 = always align
                                                2 = never align
+                                               3 = align external partitions
                                          bit10-13= System area sub type
                                                With type 0 = MBR:
                                                Gets overridden by bit0 and bit1.
@@ -410,6 +459,7 @@ struct XorrisO { /* the global context of xorriso */
                                                    any other feature which
                                                    needs to have own MBR
                                                    partition entries.
+                                               2 = generic MBR
                                          bit14= Only with System area type 0
                                                GRUB2 boot provisions:
                                                Patch system area at byte 92 to
@@ -418,11 +468,17 @@ struct XorrisO { /* the global context of xorriso */
                                                Little-endian 8-byte.
                                                Should be combined with
                                                options bit0.
+                                         bit15= Only with MBR:
+                                               Enforce MBR "bootable/active"
+                                               flag. In worst case by dummy
+                                               partition of type 0x00 which
+                                               occupies block 0.
                                        */
  int patch_system_area;               /* Bits as of system_area_options.
                                          to be applied to the loaded system
                                          area of the image, if no 
-                                         system_area_disk_path is set.
+                                         system_area_disk_path and no
+                                         system_area_options are set.
                                        */
 
  /* The number of unclaimed 2K blocks before start of partition 1 as of
@@ -442,6 +498,32 @@ struct XorrisO { /* the global context of xorriso */
  /* Path and type of image files to be appended as MBR partitions */
  char *appended_partitions[Xorriso_max_appended_partitionS];
  uint8_t appended_part_types[Xorriso_max_appended_partitionS];
+ uint8_t appended_part_type_guids[Xorriso_max_appended_partitionS][16];
+ /* Flags in case that appended partitions show up in GPT:
+    bit0= appended_part_type_guids is valid
+ */
+ uint8_t appended_part_gpt_flags[Xorriso_max_appended_partitionS];
+ /* If 1: With appended partitions: create protective MBR and mark by GPT */
+ int appended_as_gpt;
+ /* If 1: With appended partitions: mark by APM */
+ int appended_as_apm;
+
+ /* If 1: Apply isohybrid gestures to non-isohybrid situations */
+ int part_like_isohybrid;
+
+ /* MBR partition type of ISO filesystem partition or protective partition
+    if not real GPT or CHRP.
+ */
+ int iso_mbr_part_type;
+ uint8_t iso_gpt_type_guid[16];
+ int iso_mbr_part_flag;
+ /* Flags in case that the partition table is GPT:
+    bit0= iso_gpt_type_guid is valid
+ */
+
+ /* See libisoburn.h isoburn_igopt_set_gpt_guid() */
+ uint8_t gpt_guid[16];
+ int gpt_guid_mode;
 
  /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
  */
@@ -470,6 +552,16 @@ struct XorrisO { /* the global context of xorriso */
  /* To eventually override vol_modification_time by unconverted string
     and timezone 0 */
  char vol_uuid[17];
+ /* To flatly set all file timestamps to the same value
+    Special value: "set_to_mtime", see -find action "set_to_mtime"
+  */
+ char all_file_dates[80];
+
+ /* Timestamp for ISO files which have no origin file on disk and thus
+    normally get time(NULL) as timestamps
+ */
+ int do_override_now_time;
+ time_t now_time_override;
 
 #ifdef Xorriso_with_libjtE
  /* Parameters and state of Jigdo Template Export environment */
@@ -492,7 +584,11 @@ struct XorrisO { /* the global context of xorriso */
  char loaded_boot_cat_path[SfileadrL];
 
  /* XORRISO options */
- int allow_graft_points;
+ int allow_graft_points;   /* if not zero: yes
+                              special options:
+                              bit1= unescape '\\' too,
+                                    continue unescaping after first real '='
+                            */
 
  int allow_restore; /* -2=disallowed until special mode "unblock"
                        -1=permanently disallowed
@@ -559,7 +655,7 @@ struct XorrisO { /* the global context of xorriso */
  int structured_search;
  /* 0= flat text search
     1= '/' is a significant separator that cannot be matched by wildcards
-    ( 2= like 1 : but report only occurence in tree, no payload, no location )
+    ( 2= like 1 : but report only occurrence in tree, no payload, no location )
     ( 3= like 2 : but report first content level of matching directories )
     4= actually not structured but unique find mode (with search_mode 4)
  */
@@ -570,13 +666,14 @@ struct XorrisO { /* the global context of xorriso */
  int temp_mem_limit;
 
  off_t file_size_limit;
+ int file_name_limit;
 
  struct ExclusionS *disk_exclusions;
  int disk_excl_mode; /* bit0= on (else off)
                         bit1= parameter too (else rekursion only)
                         bit2= whole subtree banned (else only exact path)
                         bit3= when comparing ignore excluded files rather
-                              than to treat them as truely missing on disk
+                              than to treat them as truly missing on disk
                      */
 
  struct ExclusionS *iso_rr_hidings;
@@ -699,6 +796,10 @@ struct XorrisO { /* the global context of xorriso */
                                bit2= seen data file without MD5
                                bit3= seen match
                              */
+ int find_unique_trunc_result; /* 0= some names are not uniquely truncatable
+                                  1= all names are uniquely truncatable
+                                  2= no names need truncation
+                                */
 
  double last_abort_file_time; /* most recent check for aborting -check_md5 */
 
@@ -735,6 +836,15 @@ struct XorrisO { /* the global context of xorriso */
  */
  int update_flags;
 
+ /* For find jobs show_hfs_cmd , show_hfs_as_mkisofs 
+ */
+ /* bit0= do not record but only count
+    bit1= as_mkisofs mode
+ */
+ int show_hfs_cmd_flag;
+ int show_hfs_cmd_count;
+ char **show_hfs_cmds;
+
  /* result (stdout, R: ) */
  char result_line[10*SfileadrL];
  int result_line_counter;
