@@ -4,7 +4,7 @@
    a command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2018 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2019 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  5
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  2
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -151,6 +151,12 @@ int Xorriso_ls_filev(struct XorrisO *xorriso, char *wd,
 */
 int Xorriso_ls(struct XorrisO *xorriso, int flag);
 
+/*
+   @return: <=0 = error, 1= directory tree, 2= path leads to non-directory
+*/
+int Xorriso_get_dus(struct XorrisO *xorriso, char *iso_rr_path, off_t *size,
+                    off_t boss_mem, int flag);
+
 /* @param wd        Path to prepend in case img_path is not absolute
    @param img_path  Absolute or relative path to be normalized
    @param eff_path  returns resulting effective path.
@@ -642,12 +648,20 @@ int Xorriso_truncate_path_comps(struct XorrisO *xorriso, char *path,
 int Xorriso_graftable_pathspec(struct XorrisO *xorriso, char *in_pathspec,
                                char *pathspec, int flag);
 
+int Xorriso_parse_guid(struct XorrisO *xorriso, char *text,
+                       uint8_t guid[16], int flag);
+
 int Xorriso_parse_gpt_guid(struct XorrisO *xorriso, char *text, int flag);
 
+int Xorriso_parse_type_guid(struct XorrisO *xorriso, char *text,
+                            uint8_t guid[16], int *mbr_type, int flag);
+
 int Xorriso_format_guid(struct XorrisO *xorriso, uint8_t guid[16], char *line,
                         int flag);
 
 int Xorriso_make_guid(struct XorrisO *xorriso, char *line, int flag);
 
+int Xorriso_set_libisofs_now(struct XorrisO *xorriso, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
